﻿Set-ExecutionPolicy unrestricted

#Help

#Include details
help New-AzureVM -Detailed

#Open online help
help Get-Process -online

#Show examples
help New-AzureVM -Examples

#Open in external window
help New-AzureVm -ShowWindow #Power Tip!

#Find correct cmdlet
help *Azure*VM*

#Open Window for Parameters
Show-Command New-AzureVM     #Power Tip!


#Objects

get-date
(get-date).GetType();
get-date | Get-Member

(get-date).ToShortDateString();
(get-date).ToString("s");

#pipelining

Find Alias: help Get-Process
Get-Process | Get-Member

Get-Process | Sort-Object -Property CPU -Descending | Select-Object -First 5 -Property ProcessName, cpu, virtualmemorysize
ps | sort -Pr CPU -desc | select -f 5 -pr ProcessName, cpu, virtualmemorysize


Get-Counter '\Processor(_total)\% Processor Time' -Continuous -SampleInterval 1


#output
$proc = Get-Process | Sort-Object WS -Descending | Select-Object -First 15

$proc
$proc | Format-Table name, ws, id 
$proc | Format-Table name, ws, id -Autosize
$proc | format-List
$proc | Format-List * 
$proc | Out-GridView #Power Tip!
$proc | Out-File D:\__TEMP\out.txt
ii . (invoke-item)
$proc > text.txt
ii text.txt
$proc | Export-Csv D:\__TEMP\outcsv.txt
$proc | ConvertTo-Html > c:\text.html
$proc | ConvertTo-Json
$proc | clip #(Copy to clipboard)
start explorer.exe D:\__TEMP\out.txt
start explorer.exe D:\__TEMP\outcsv.txt






#Providers
Get-PSProvider
cd hklm:
cd cert:


#WMI
get-command *WMI* -Type Cmdlet
Get-WmiObject -Class * -List

Get-WmiObject -list win32*video*
Get-WmiObject win32_startupcommand





#COM Interop SPEECH
$s = New-Object -ComObject SAPI.SPVoice
$s.Speak("Hey EMSUGS")

$text = get-content c:\file.txt
$s.speak($text);

#--> Demo invoke Speech
#--> Demo outlook.com login
#--> Demo Excel Interop


#.NET

#BEEP
[System.Media.SystemSounds]::Beep.Play()  
[console]::beep(500,500)

#Show MessageBox
[System.Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms") 
[System.Windows.Forms.MessageBox]::Show("Powershell at MSUGS, Yay!")

#--> Msugs2. Balloon.
#--> WPFGui1
#--> WPFGui2
#--> WPFOSDetails
#--> WPFClockWidget



#WebServices w. intellisense
#-->msugs2!
$uri = 'http://www.webservicex.net/geoipservice.asmx?WSDL'
$g = New-WebServiceProxy -Uri $uri -Namespace ws
$g.GetGeoIP('213.131.66.246');
$g.GetGeoIPContext();


#Stock Quote WS
#-->msugs2!
$uri = 'http://www.webservicex.net/stockquote.asmx?WSDL'
$proxy = New-WebServiceProxy -Uri $uri -Namespace ws
$proxy.GetQuote('MSFT')

[XML] $stock = $proxy.GetQuote('MSFT')

$stock.InnerXml
$stock.StockQuotes.Stock


#inspecting Web Requests
$web = Invoke-WebRequest http://www.msugs.ch

$web.BaseResponse
$web.BaseResponse.Cookies
$web.Images | Select-Object src
$web.Links | select -Property innerText, href





#--> PSInvaders





#doesnt work!!!
#REST APIs
$uri = "https://gdata.youtube.com/feeds/api/videos?v=2"
$query = "&q=" + "powershell"
Invoke-RestMethod "$uri$query";


Invoke-RestMethod "$uri$query" | ForEach-Object {
    [PSCustomObject]@{
        Title = $_.Title
        Author = $_.Author.name
        URL = $_.content.src
        Term = $_.category.term[1]
    }
} | format-list